# -*- coding: utf-8 -*-
##############################################################################
#
#    OERPEnv, OpenERP Environment Administrator
#    Copyright (C) 2011-2015 Coop Trab Moldeo Interactive 
#    (<http://www.moldeointeractive.com.ar>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from os.path import join
import sys, os, ConfigParser
import subprocess
import psycopg2

# Use 1 to postgresql v8.x
# Use 3 to postgresql v9.x
default_isolation_level=3

class PostgresNotRunningError(RuntimeError):
    def __init__(self):
        self.message = "No postgresql server to connect.\nPostgreSQL server is not running or other server is locking the database."

def create_database(dbname):
        """
        Create a postgresql database.
        """
        conn = psycopg2.connect("")
        old_isolation_level = conn.isolation_level
        conn.set_isolation_level(0)
        cur = conn.cursor()
        try:
                cur.execute("CREATE DATABASE %s;" % dbname)
        except psycopg2.ProgrammingError:
                cur.execute("DROP DATABASE %s;" % dbname)
                cur.execute("CREATE DATABASE %s;" % dbname)
        conn.set_isolation_level(old_isolation_level)

def drop_database(dbname):
        """
        Drop a postgresql database.
        """
        import psycopg2

        conn = psycopg2.connect("")
        old_isolation_level = conn.isolation_level
        conn.set_isolation_level(0)
        cur = conn.cursor()
        cur.execute("DROP DATABASE %s;" % dbname)
        conn.set_isolation_level(old_isolation_level)

def save_configuration(options, filename):
        """
        Save a configuration file with options dict data.
        """
        try:
            p = ConfigParser.ConfigParser()
            sections = {}
            for o in options.keys():
                if not len(o.split('.'))==2:
                    continue
                osection,oname = o.split('.')
                if not p.has_section(osection) and osection != 'DEFAULT':
                    p.add_section(osection)
                p.set(osection,oname,options[o])
            p.write(file(filename,'wb'))
        except:
            print 'Unable to write config file %s !'% (filename,)
        return True

def load_configuration(filename, defaults=None):
        """
        Return a dictonary from a ConfigParse format file.
        """
        options = {}
        try:
            p = ConfigParser.ConfigParser(defaults=defaults)
            p.read([filename])
            for section in p.sections():
                for (name,value) in p.items(section):
                    if value=='True' or value=='true':
                        value = True
                    if value=='False' or value=='false':
                        value = False
                    if value=='None' or value=='none':
                        value = None
                    options[section+'.'+name] = value
        except Exception, e:
            raise RuntimeError('Unable to read config file %s !: %s'% (filename, e))
        return options

def recover_snapshot(dbname, snapshot, oerpenv):
    try:
        #create_database(dbname);
        infile = join(oerpenv.snapshots_path, "%s_%s.dump" % (dbname, snapshot))
        P = subprocess.Popen(['pg_restore', '-x', '-c', '-O', '--disable-triggers', '-Fc', '-d', dbname, infile])
        r = P.wait()
        if r:
            return False
        return True
    except psycopg2.OperationalError:
        raise PostgresNotRunningError

def exists_db(dbname):
    """
    Check if exits a postgresql database.
    """
    try:
        conn = psycopg2.connect("")
        cur = conn.cursor()
        cur.execute("SELECT datname FROM pg_database where datname=%s;", (dbname,))
        return bool(cur.rowcount)
    except:
        raise PostgresNotRunningError


# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:



# -*- coding: utf-8 -*-
##############################################################################
#
#    OERPEnv, OpenERP Environment Administrator
#    Copyright (C) 2011-2015 Coop Trab Moldeo Interactive 
#    (<http://www.moldeointeractive.com.ar>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from os.path import abspath, basename, dirname, join, exists, lexists, realpath
import os

class Addon:
        def __init__(self, addon_config_path):
            """
            Init an addon class information 
            """
            self.config_path = abspath(addon_config_path)
            self.path = dirname(self.config_path)
            self._token = basename(self.path)
            self._description = None

        def read_description(self):
            """
            Read Addon file description.
            """
            if self._description is None:
                    self._description = eval(open(self.config_path).read())

        @property
        def token(self):
            """
            Return addon name.
            """
            return self._token

        @property
        def name(self):
            """
            Return addon short description.
            """
            self.read_description()
            return self._description['name']

        @property
        def description(self):
            """
            Return addon long description.
            """
            self.read_description()
            return self._description.get('description','')

        @property
        def depends(self):
            """
            Return addon list with the addon depends.
            """
            self.read_description()
            return self._description.get('depends',[])

        @property
        def website(self):
            """
            Return the website of the addon.
            """
            self.read_description()
            return self._description.get('website',None)

        @property
        def author(self):
            """
            Return the author of the addon.
            """
            self.read_description()
            return self._description.get('author',None)

        @property
        def version(self):
            """
            Return the version of the addon.
            """
            self.read_description()
            return self._description.get('version',None)

        def environment_path(self, environment):
            """
            Return the path of the addon in the environment.
            """
            addons_path = environment.get_addonsourcepath()['server']
            return join(addons_path, self.token)

        def is_enable(self, environment):
            """
            Return true if the addon is enabled in the environment.
            """
            path = self.environment_path(environment)
            return exists(path) and \
                realpath(path) == self.path

        def is_saned(self, environment):
            """
            Return true if the addon is saned.
            """
            path = self.environment_path(environment)
            return lexists(path) and os.path.exists(realpath(path)) or not lexists(path)

        def enable(self, environment, force=False):
            """
            Enable this addon in this environment. Not check depends.
            """
            addons_path = environment.get_addonsourcepath()['server']
            where_install = join(addons_path, self.token)
            is_enabled = self.is_enable(environment)
            is_saned = self.is_saned(environment)
            is_exists = os.path.exists(where_install)
            is_link = os.path.islink(where_install)

            if is_link:
                if ((is_exists and force) or \
                    (is_enabled and force) or \
                    (not is_saned)):
                    os.remove(where_install)
                elif is_enabled and not force:
                    return False

            if not os.path.exists(where_install):
                os.symlink(self.path, where_install)

            return True

        def disable(self, environment, force=False):
            """
            Disable this addon in this environment. Not check depends.
            """
            if self.is_enable(environment):
                os.remove(where_install)
                return True
            elif not self.is_enable(environment) and force:
                return True
            return False

        @property
        def objects(self):
            """
            Return a duple with a list of objects declared and inherited in the addon.
            """
            import re
            objects = set()
            inherited = set()
            for filename, name, match in self.search( 
                {'object':re.compile(r'^\s*_name\s*=\s*["\']([a-z][\w\.]*)["\']'),
                 'inherit':re.compile(r'^\s*_inherit\s*=\s*["\']([a-z][\w\.]*)["\']'),},
                re.compile(r'^.*\.py$')
            ):
                if name == 'object':
                    objects.update(match)
                else:
                    inherited.update(match)
            return objects - inherited, inherited

        @property
        def entities(self):
            """
            Return a list of entities declared in xml.
            """
            import re
            record = set()
            items = {}
            for filename, name, match in self.search( 
                {'record':re.compile(r'id\s*=\s*["\']([^"]*)["\']'), },
                re.compile(r'^.*\.xml$')
            ):
                if name == 'record':
                    record.update(match)
            return record

        def entity(self, entity):
            """
            Return a entity information.
            """
            import re
            record = set()
            items = {}
            for filename, name, match in self.search( 
                {'record':re.compile(r'id\s*=\s*["\']([^"]*)["\']'), },
                re.compile(r'^.*\.xml$')
            ):
                if name == 'record' and entity in match:
                    yield filename

        def search(self, re_patterns, re_file):
            """
            Search in files
            """
            search_files = set()
            for p, ds, fs in os.walk(self.path):
                    search_files.update(set([ join(p,f) for f in fs if re_file.search(f)]))

            for filename in search_files:
                with open(filename) as file:
                    lines = file.readlines()
                    for re_name, re_pattern in re_patterns.items():
                        matchs = [ re_pattern.search(line) for line in lines ]
                        matchs = [ match.group(1) for match in matchs if match != None ]
                        if len(matchs) > 0:
                            yield filename, re_name, matchs

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:



# -*- coding: utf-8 -*-
##############################################################################
#
#    OERPEnv, OpenERP Environment Administrator
#    Copyright (C) 2011-2015 Coop Trab Moldeo Interactive 
#    (<http://www.moldeointeractive.com.ar>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from bzrlib.plugin import load_plugins
from bzrlib.branch import Branch
from bzrlib import workingtree
from os.path import abspath, basename, dirname, join, exists
import imp
import os
import re
import subprocess
import StringIO

try:
    import pysvn
    without_svn = False
except:
    without_svn = True

class RepositoryBase:
    def __init__(self, local_path, remote_url):
        self.local_path = local_path
        self.remote_url = remote_url

    def update(self):
        raise NotImplementedError

    def checkout(self):
        raise NotImplementedError

    def state(self):
        if exists(self.local_path):
            return 'exists'
        else:
            return 'no exists'

    _url_re_ = []

class BazaarRepository(RepositoryBase):
    def __init__(self, local_path, remote_url):
        self = RepositoryBase.__init__(self, local_path, remote_url)
        load_plugins()

    def update(self):
        from bzrlib import workingtree
        wt = workingtree.WorkingTree.open(self.local_path)
        wt.update()
        #remote_branch = Branch.open(self.remote_url)
        #wt.pull(remote_branch)

    def checkout(self):
        #import pdb; pdb.set_trace()
        remote_branch = Branch.open(self.remote_url)
        tree = remote_branch.create_checkout(self.local_path, lightweight=True)
        #local_branch = remote_branch.bzrdir.sprout(
        #        self.local_path).open_branch()

    _url_re_ = [
        re.compile('^lp:.*$'),
        re.compile('^bzr+ssh:.*')
    ]

def ssl_server_trust_prompt( trust_dict ):
    # http://pysvn.tigris.org/docs/pysvn_prog_ref.html#pysvn_client_callback_ssl_server_trust_prompt
    for v in trust_dict.items():
       print "%s:%s" % v
    a = raw_input("Your trust in the certificate? ")
    return a in 'Yesyes', trust_dict['failures'], True

class SVNRepository(RepositoryBase):
    def __init__(self, local_path, remote_url):
        if without_svn:
            raise RuntimeError("You need install PySVN to use SVN capabilities.")

        self = RepositoryBase.__init__(self, local_path, remote_url)
        load_plugins()

    def update(self):
        client = pysvn.Client()
        client.callback_ssl_server_trust_prompt = ssl_server_trust_prompt
        client.update(self.local_path)

    def checkout(self):
        client = pysvn.Client()
        client.callback_ssl_server_trust_prompt = ssl_server_trust_prompt
        client.checkout(self.remote_url, self.local_path)

    _url_re_ = [
        re.compile('^https:.*$'),
        re.compile('^svn:.*$'),
        re.compile('^svn+ssh:.*'),
    ]

class GITRepository(RepositoryBase):
    def __init__(self, local_path, remote_url):
        self = RepositoryBase.__init__(self, local_path, remote_url)

    def update(self):
        olddir = os.getcwd()
        os.chdir(self.local_path)
        subprocess.call(['git', 'pull'])
        os.chdir(olddir)

    def checkout(self):
        subprocess.call(['git', 'clone', self.remote_url, self.local_path])

    _url_re_ = [
        re.compile('^git@.*$'),
    ]

def Repository(local_path, branch_url):
    classes = [ BazaarRepository, SVNRepository, GITRepository ]
    for c in classes:
        if any([ ure.search(branch_url) is not None for ure in c._url_re_ ]):
            return c(local_path, branch_url)
        
# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
 



# -*- coding: utf-8 -*-
##############################################################################
#
#    OERPEnv, OpenERP Environment Administrator
#    Copyright (C) 2011-2015 Coop Trab Moldeo Interactive 
#    (<http://www.moldeointeractive.com.ar>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

import re
import virtualenv
import sys
import time
from os import makedirs, walk, symlink
from os.path import abspath, join, exists, dirname, basename
from installable import Installable
from oerpenv import tools, defaults
from virtualenv import subprocess
from addon import Addon
from repository import Repository
from pwd import getpwnam

class NoEnvironmentConfigFileError(RuntimeError):
    def __init__(self, filepath):
        self.filepath = filepath
        self.message = "No environment config file available (%s)" % filepath

class NoVersionAvailableError(RuntimeError):
    def __init__(self, version):
        self.version = version
        self.message = "No version available (%s)" % version

class OpenERPEnvironment:
    def __init__(self, sources=None, config_filename=defaults.config_filename, init=False, version='6.0'):
        """
        Create an instance of OpenERPEnvironment class.

        Arguments:
        config_filename -- Full path to the configuration file.
        """
        if not sources is None and not exists(sources):
            makedirs(sources)
        if exists(config_filename):
            self.config_filename = abspath(config_filename)
            self.root_path = dirname(self.config_filename)
        elif init:
            self.config_filename = abspath(config_filename)
            self.root_path = dirname(self.config_filename)
            if not exists(self.root_path):
                makedirs(self.root_path)
            self._config = defaults.environment_configuration
            if not version in defaults.version_configuration:
                raise NoVersionAvailableError(version)
            self._config.update(defaults.version_configuration[version])
            self._config['Environment.root'] = self.root_path
            if not sources is None:
                default_sources = self._config['Environment.sources']
                symlink(sources, default_sources)
                self._config['Environment.sources'] = sources
            self.environments = []
            self.create_python_environment(defaults.python_environment)
            self.save(init=True)
            for d in defaults.directory_structure:
                nd = join(self.root_path, d)
                if not exists(nd):
                    makedirs(nd)
        else:
            raise NoEnvironmentConfigFileError(config_filename)

        self.load()
        self.defaults = {}
        self.config_filename = join(self.root_path, self.config_filename)
        self.set_python_environment(self.environments[0])
        self.addonsourcepath=None

    def changetouser(self):
        """
        Change effective user to run this process
        """
        username = self._config['Environment.user']
        uid = getpwnam(username)[2]

    def load(self):
        """
        Load configuration file.
        """
        self._config = tools.load_configuration(self.config_filename, defaults={'root': self.root_path})
        self.environments = self._config['Environment.environments'].split(',')

    def save(self, init=False):
        """
        Save configuration file.
        """
        if not init:
            self._config['Environment.environments'] = ','.join(self.environments) 
        tools.save_configuration(self._config, self.config_filename)

    def update(self, iterate=False, repositories=[]):
        """
        Update sources.

        Arguments:
        iterate -- If true the update the function yield to an iteration.
        """
        for name, repository in self.repositories.iteritems():
            if repository is None:
                raise RuntimeError('Repository %s is not exists' % name)
            if len(repositories) == 0 or name in repositories:
                if repository.state() == 'no exists':
                    command = 'create'
                else:
                    command = 'update'
                if iterate:
                    yield command, repository.local_path, repository.remote_url
                if command == 'create':
                    repository.checkout()
                else:
                    repository.update()

    def create_python_environment(self, name):
        """
        Create a new python environment with name
        """
        path = join(self.root, name)
        if exists(path): return False
	virtualenv.logger = virtualenv.Logger([(virtualenv.Logger.level_for_integer(2), sys.stdout)])
        virtualenv.create_environment(path,site_packages=False)
        if name not in self.environments:
                self.environments.append(name)

        client_config_filename=join(path, 'etc', self._config['Environment.client-config-filename'])

        if not exists(dirname(client_config_filename)):
            makedirs(dirname(client_config_filename))

        if not exists(client_config_filename):
            tools.save_configuration(defaults.options_client_configuration(path),
                                     client_config_filename)

        self.save()
        return True

    def add_repository(self, branch_name, branch_url):
        self._config['Repositories.%s' % branch_name] = branch_url

    @property
    def binary_path(self):
        return join(self.env_path, 'bin')

    @property
    def installables(self):
        installables_str = self._config.get('Environment.installables', False)
        if installables_str:
            for i in re.split('\s+', installables_str):
                if len(i)>1:
                    method, url = i.strip().split(':',1)
                    yield Installable(method, url, join(self.env_path,'bin'))
        else:
            raise StopIteration

    @property
    def modules(self):
        return [ Installable(m, is_application=False) for m in self._config['Environment.modules'].split(',') ]

    def addons(self, token_filter=None, object_filter=None, inherited_filter=None, entity_filter=None):
        if not token_filter is None:
            filter_re = re.compile(token_filter)
            filter_t = lambda s: filter_re.search(s) != None
        else:
            filter_t = lambda s: True

        if not object_filter is None:
            filter_o = lambda a: object_filter in a.objects[0]
        else:
            filter_o = lambda a: True

        if not inherited_filter is None:
            filter_i = lambda a: inherited_filter in a.objects[1]
        else:
            filter_i = lambda a: True

        if not entity_filter is None:
            filter_e = lambda a: entity_filter in a.entities
        else:
            filter_e = lambda a: True

        for path, ds, fs in walk(self.sources_path, followlinks=True):
            if self._config['Environment.desc-filename'] in fs and \
               '__init__.py' in fs and \
               filter_t(basename(path)):
                addon = Addon(join(path, self._config['Environment.desc-filename']))
                if filter_o(addon) and filter_i(addon) and filter_e(addon):
                    yield addon
                ds = []

    @property
    def version(self):
        return self._config['Environment.version']

    @property
    def repositories(self):
        return dict([ (name.split('.')[1], Repository(join(self.sources_path, name.split('.')[1]), remote_source))
                 for name, remote_source in self._config.items()
                 if name.split('.')[0] == 'Repositories' and name.split('.')[1] != 'root'])

    @property
    def root(self):
        return self._config.get('Environment.root', self.root_path)

    @property
    def sources_path(self):
        return self._config['Environment.sources']

    @property
    def description_filename(self):
        return self._config['Environment.desc-filename']

    @property
    def reports_path(self):
        return self._config['Environment.reports']

    @property
    def snapshots_path(self):
        return self._config['Environment.snapshots']

    @property
    def addons_dir(self):
        return self._config['Environment.addons']

    @property
    def desc_filename(self):
        return self._config['Environment.desc-filename']

    @property
    def client_config_filename(self):
        return join(self.env_path, 'etc', self._config['Environment.client-config-filename'])

    @property
    def server_config_filename(self):
        if 'Environment.server-config-filename' in self._config:
            return join(self.env_path, 'etc', self._config['Environment.server-config-filename'])
        else:
            False

    @property
    def web_config_filename(self):
        if 'Environment.web-config-filename' in self._config:
            return join(self.env_path, 'etc', self._config['Environment.web-config-filename'])
        else:
            False

    @property
    def extracommands(self):
        return self._config.get('Environment.server-extracommands', None)

    @property
    def production(self):
        return self._config.get('Environment.production', False)

    @property
    def language(self):
        return self._config.get('Environment.language', None)

    @property
    def snapshot(self):
        return self._config.get('Database.snapshot', False)

    @property
    def database(self):
        return self._config.get('Database.database', None)

    @property
    def debug(self):
        return self._config.get('Environment.debug', False)

    @property
    def modules_update(self):
        r = self._config.get('Modules.update', '')
        r = [ s for s in r.split(',') if not s == '' ]
        return r

    @property
    def modules_install(self):
        r = self._config.get('Modules.install', '')
        r = [ s for s in r.split(',') if not s == '' ]
        return r

    def set_python_environment(self, name):
        """
        Set as the python environment where openerp will run.
        """
        self.py_environment_name = name
        self.env_path = join(self.root_path, self.py_environment_name)

    def execute(self, command, args, no_wait=False, check_for_termination=False):
        """
        Execute a command in the python environment defined in set_python_environment()
        """
        if no_wait:
            P = subprocess.Popen([join(self.env_path,'bin',command)] + args)
            if check_for_termination:
                time.sleep(5)
                if not P.poll() is None: return None
            return P.pid
        else:
            P = subprocess.Popen([join(self.env_path,'bin',command)] + args)
            P.wait()
            return None

    def get_addonsourcepath(self):
        """
        Return a list of path to addons directories.
        """
        if self.addonsourcepath is not None:
            return self.addonsourcepath

        python_exe = join(self.env_path, 'bin', 'python')

        _query_addons = {
            '6.0': """\
import pkg_resources, os.path
TE = pkg_resources.Environment()
print pkg_resources.resource_filename('openerp-server', 'addons')
print os.path.join(TE["openerp-web"][0].location,'addons')
            """,
            '6.1': """\
import pkg_resources, os.path
TE = pkg_resources.Environment()
print pkg_resources.resource_filename('openerp', 'addons')
print os.path.join(TE["openerp-web"][0].location,'addons')
            """,
            '7.0': """\
import pkg_resources, os.path
TE = pkg_resources.Environment()
print pkg_resources.resource_filename('openerp', 'addons')
if len(TE["openerp-web"])>0: print os.path.join(TE["openerp-web"][0].location,'addons')
            """,
            '8.0': """\
import pkg_resources, os.path
TE = pkg_resources.Environment()
print pkg_resources.resource_filename('openerp', 'addons')
if len(TE["openerp-web"])>0: print os.path.join(TE["openerp-web"][0].location,'addons')
            """,
        }

        p = subprocess.Popen([ python_exe, '-c', _query_addons[self.version] ],
                            stdout=subprocess.PIPE, stdin=subprocess.PIPE)
        addons_path = p.stdout.readlines()
        addons_path = dict(zip(['server', 'web'],[ p.strip() for p in addons_path if exists(p.strip()) ]))
        self.addonsourcepath = addons_path
        return addons_path

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:



# -*- coding: utf-8 -*-
##############################################################################
#
#    OERPEnv, OpenERP Environment Administrator
#    Copyright (C) 2011-2015 Coop Trab Moldeo Interactive 
#    (<http://www.moldeointeractive.com.ar>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from os.path import abspath, basename, dirname, join, exists
import imp
import os
import re
import subprocess
import StringIO

class Installable:
    def __init__(self, method, url, bin_path):
        """
        Init an addon class information 
        """
        self._method = method
        repository_type = None
        if exists(url):
            repository_type = 'file'
        if not repository_type and '+' in url and method in ['pip']:
            repository_type, url = url.split('+',1)
        if not repository_type and not '+' in url and ':' in url and method in ['pip']:
            repository_type = url.split(':',1)[0]
        self._url = url
        self._repository_type = repository_type
        self._bin_path = bin_path
        self._run = { 'setup': self.run_setup, 'pip': self.run_pip }[method]
        self._short_name = re.split(r'[<>=]+', url, 1)[0]
        self._name = None
        self._fullname = None
        self._description = None

    def run_setup(self, command, wait=True):
        bin_path = self._bin_path
        url = self._url
        command = [ join(bin_path, 'python'), join(url,'setup.py'), command ]
        P = subprocess.Popen(command,
                             cwd = url, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        out = P.stdout.readlines()
        err = P.stderr.readlines()
        if wait:
            r = P.wait()
            return out, err, r
        return out, err, r, P.wait

    def run_pip(self, command):
        bin_path = self._bin_path
        url = self._url
        if command in ['install']:
            url = "%s+%s" % (self._repository_type,url) if not self._repository_type in [None, 'file', 'http', 'https'] else url
        if command in ['search']:
            url = self._short_name
        command = [ join(bin_path, 'pip'), command, url ]
        P = subprocess.Popen(command,
                             stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        out = P.stdout.readlines()
        err = P.stderr.readlines()
        r = P.wait()
        return out, err, r

    def read_description(self):
        """
        Read Installable description.
        """
        method = self._method
        url = self._url

        name = url 
        fullname = 'No fullname'
        description = 'No description'

        if method in ['setup', 'pip'] and self._repository_type in ['file']:
            name, err, r = self.run_setup('--name')
            if r != 0:
                print "ERROR: This package is not well configured, or you need install previous modules to make it work"
                print err
            name=','.join(name).strip() if r == 0 else url 

            fullname, err, r = self.run_setup('--fullname')
            fullname=','.join(fullname).strip() if r == 0 else 'No fullname'

            description, err, r = self.run_setup('--description')
            description=','.join(description).strip() if r == 0 else 'No description'

        elif method in ['pip'] and not self._repository_type in ['hg', 'bzr', 'file', 'http', 'https']:
            out, err, r = self.run_pip('search')
            try:
                outs = dict([
                    (lambda(a,b): (a.strip().lower(), b))(re.split(r'\s*-\s+',i,1))
                    for i in out if '- ' in i ])
            except:
                outs = {}
            if self._short_name.lower() in outs:
                name = self._short_name
                fullname = 'No fullname'
                description = outs[self._short_name.lower()]
            else:
                raise RuntimeError('Installable %s not found\nWe found only this options:\n%s' % (url, ''.join(out)))
        elif not self._repository_type in [ 'hg', 'bzr', 'http', 'https' ]:
            raise RuntimeError('Method not supported or wrong config file.')

        self._name = name
        self._fullname = fullname
        self._description = description

    def install(self):
        out, err, r = self._run('install')

        if (r == 0):
            return True
        else:
            print ''.join(out)
            print '\n'.join(err)

        return False

    @property
    def path(self):
        return self._path

    @property
    def name(self):
        if self._name is None: self.read_description()
        return self._name

    @property
    def fullname(self):
        if self._fullname is None: self.read_description()
        return self._fullname

    @property
    def description(self):
        if self._description is None: self.read_description()
        return ''.join(self._description)

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:



#!/usr/bin/env python
##############################################################################
#
#    OERPEnv, OpenERP Environment Administrator
#    Copyright (C) 2011-2015 Coop Trab Moldeo Interactive 
#    (<http://www.moldeointeractive.com.ar>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from setuptools import setup

setup(name='oerpenv',
      version='1.8.7',
      author='Cristian S. Rocha',
      author_email='cristian.rocha@moldeo.coop',
      maintainer='Cristian S. Rocha',
      maintainer_email='cristian.rocha@moldeo.coop',
      url='http://business.moldeo.coop/',
      description='OpenERP Environment Administrator',
      long_description="""
      OERPenv helps you manage virtual python environments with different OpenERP servers.
      It's make easy to develop, migrate and maintain different versions of OpenERP servers in the same hardware box.

      This package installs command 'oerpenv' with these sub-commands:

      $ oerpenv -h
      usage: oerpenv [-h]
                     
                     {restore,web-stop,show,server-start,activate,pip,shell-db,web-start,create,add,init,server-stop,create-db,test,search-entity,init-db,enable,csv2xml,update,disable,search-object,list-installables,dummy,drop-db,setup,list-addons,client,list-db,install,snapshot}
                     ...

      optional arguments:
        -h, --help            show this help message and exit

      subcommands:
        The OpenERP environment administrator help you to administrate OpenERP
        environments. You can use the following commands.

        {restore,web-stop,show,server-start,activate,pip,shell-db,web-start,create,add,init,server-stop,create-db,test,search-entity,init-db,enable,csv2xml,update,disable,search-object,list-installables,dummy,drop-db,setup,list-addons,client,list-db,install,snapshot}
                              commands
          init                Init an environment in the work path or in the
                              declared path.
          setup               Init an environment in the work path or in the
                              declared path.
          update              Update sources.
          create              Create a new python environment.
          add                 Add a branch with to the sources list.
          install             Install all software in the default environment of in
                              the declared.
          list-installables   List all availables applications in sources.
          list-addons         List availables addons in sources. Show all addons if
                              not filter expression declared.
          enable              Enabel addons on the environment. Create a symbolic
                              link.
          disable             Disable addons on the environment. Remove a symbolic
                              link.
          dummy               Create a dummy addon. Useful to create new addon.
          test                Execute the server in test mode for this addon.
          client              Execute the server in test mode for this addon.
          server-start        Start the server.
          server-stop         Stop the server.
          web-start           Start the web client.
          web-stop            Stop the web client.
          search-object       Search addons with this object.
          search-entity       Search in xml some declared entity with id
          show                Show addon information.
          list-db             List availables databases.
          shell-db            Execute a shell for sql commands over the database.
          create-db           Create a void database.
          drop-db             Remove a database.
          init-db             Prepare a minimalistic OpenERP database.
          snapshot            Generate a database snapshot.
          restore             Restore a database snapshot.
          pip                 Install Python packages in the virtual environment.
          csv2xml             Convert CSV files to an XML files.
          activate            Active bash instance in virtual environment.

      """,
      classifiers=[
          'Development Status :: 5 - Production/Stable',
          'Environment :: Console',
          'Intended Audience :: Developers',
          'License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)',
          'Natural Language :: English',
          'Operating System :: Unix',
          'Programming Language :: Python :: 2.6',
          'Topic :: Software Development :: Build Tools',
      ],
      scripts=['scripts/oerpenv'],
      packages=['oerpenv'],
      test_suite='tests',
      install_requires=['virtualenv','psycopg2','argparse','bzr'],
      dependency_links=['http://pysvn.barrys-emacs.org/source_kits/pysvn-1.7.5.tar.gz'],
   )

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:



# -*- coding: utf-8 -*-
##############################################################################
#
#    OERPEnv, OpenERP Environment Administrator
#    Copyright (C) 2011-2015 Coop Trab Moldeo Interactive 
#    (<http://www.moldeointeractive.com.ar>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from os.path import join
import getpass

config_filename = 'environment.conf'
version = '6.0'
python_environment = 'default'
directory_structure = ['sources', 'reports', 'snapshots', 'etc']

user = getpass.getuser()

environment_configuration = {
    'Environment.client-config-filename': 'openerp-client.conf',
    'Environment.web-config-filename': 'openerp-web.cfg',
    'Environment.server-config-filename': 'openerp-server.conf',
    'Environment.sources': '%(root)s/sources',
    'Environment.reports': '%(root)s/reports',
    'Environment.snapshots': '%(root)s/snapshots',
    'Environment.environments': 'default',
}

version_configuration = {
        '5.0': {
            'Environment.version': '5.0',
            'Environment.desc-filename': '__terp__.py',
            'Repositories.server': 'lp:~openerp/openobject-server/5.0',
            'Repositories.client': 'lp:~openerp/openobject-client/5.0',
            'Repositories.addons': 'lp:~openerp/openobject-addons/5.0',
            'Repositories.addons-extra': 'lp:~openerp-commiter/openobject-addons/stable_5.0-extra-addons',
            'Repositories.web': 'lp:~openerp/openobject-client-web/5.0',
        },
        '6.0': {
            'Environment.version': '6.0',
            'Environment.desc-filename': '__openerp__.py',
            'Environment.openerp-web.options': '--server-port=8069 --server-host=localhost --addons=default/addons/ -p 8080',
            'Environment.installables': """
                    pip:bzr+http://download.gna.org/pychart/bzr-archive
                    pip:lxml
                    pip:psycopg2
                    pip:Babel
                    pip:PyYAML
                    pip:reportlab
                    pip:python-dateutil==1.5
                    pip:hg+https://bitbucket.org/johnmc/zkemapi
                    pip:%(sources)s/server
                    setup:%(sources)s/web
            """,
            'Repositories.server': 'lp:~openerp/openobject-server/6.0',
            'Repositories.client': 'lp:~openerp/openobject-client/6.0',
            'Repositories.addons': 'lp:~openerp/openobject-addons/6.0',
            'Repositories.addons-extra': 'lp:~openerp-commiter/openobject-addons/extra-6.0',
            'Repositories.addons-community': 'lp:~openerp-community/openobject-addons/trunk-addons-community',
            'Repositories.web': 'lp:openobject-client-web',
        },
        '6.1': {
            'Environment.version': '6.1',
            'Environment.desc-filename': '__openerp__.py',
            'Environment.installables': """
                    pip:bzr+http://download.gna.org/pychart/bzr-archive
                    pip:lxml
                    pip:psycopg2
                    pip:Babel>=0.9.6
                    pip:simplejson>=2.0.6
                    pip:python-dateutil>=1.4.1,<2
                    pip:python-ldap
                    pip:python-openid
                    pip:werkzeug
                    pip:pywebdav
                    pip:unittest2
                    pip:mock
                    pip:pyyaml
                    pip:reportlab
                    pip:PIL
                    pip:gdata
                    pip:pydot
                    pip:mako
                    pip:psycopg2
                    pip:feedparser
                    pip:zsi
                    pip:pytz
                    pip:vatnumber
                    pip:vobject
                    pip:xlwt
                    pip:pyopenssl
                    pip:PyXML>=0.8.3
                    pip:python-dateutil==1.5
                    pip:hg+https://bitbucket.org/johnmc/zkemapi
                    setup:%(sources)s/server
                    setup:%(sources)s/openerp-web
            """,
            'Repositories.server': 'lp:openobject-server/6.1',
            'Repositories.addons': 'lp:openobject-addons/6.1',
            'Repositories.openerp-web': 'lp:openerp-web/6.1',
        },
        '7.0': {
            'Environment.version': '7.0',
            'Environment.desc-filename': '__openerp__.py',
            'Environment.installables': """
                    pip:bzr+http://download.gna.org/pychart/bzr-archive
                    pip:cython
                    pip:https://github.com/lxml/lxml/archive/lxml-2.3.zip
                    pip:psycopg2
                    pip:Babel
                    pip:PyYAML
                    pip:reportlab
                    pip:PIL
                    pip:python-dateutil==1.5
                    pip:hg+https://bitbucket.org/johnmc/zkemapi
                    setup:%(sources)s/server
            """,
            'Repositories.server': 'lp:openobject-server/7.0',
            'Repositories.addons': 'lp:openobject-addons/7.0',
            'Repositories.openerp-web': 'lp:openerp-web/7.0',
        },
        'trunk': {
            'Environment.version': '8.0',
            'Environment.desc-filename': '__openerp__.py',
            'Environment.installables': """
                    pip:bzr+http://download.gna.org/pychart/bzr-archive
                    pip:cython
                    pip:https://github.com/lxml/lxml/archive/lxml-2.3.zip
                    pip:psycopg2
                    pip:Babel
                    pip:PyYAML
                    pip:reportlab
                    pip:PIL
                    pip:python-dateutil==1.5
                    pip:hg+https://bitbucket.org/johnmc/zkemapi
                    pip:%(sources)s/server
                    pip:%(sources)s/openerp-web
            """,
            'Repositories.server': 'lp:openobject-server/trunk',
            'Repositories.addons': 'lp:openobject-addons/trunk',
            'Repositories.openerp-web': 'lp:openerp-web/trunk',
        },
}

openerp_header = """# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2010 Tiny SPRL (<http://tiny.be>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################
"""
import os

openerp_footer = "# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:"

options_client_configuration = lambda path: {
    'login.login': 'demo',
    'login.server': 'localhost',
    'login.port': '8070',
    'login.protocol': 'socket://',
    'login.db': 'terp',
    'client.toolbar': 'both',
    'client.theme': 'none',
    'path.share': join(path, 'share', 'openerp-client'),
    'path.pixmaps': join(path, 'share', 'pixmaps', 'openerp-client'),
    'tip.autostart': False,
    'tip.position': 0,
    'form.autosave': False,
    'printer.preview': True,
    'printer.softpath': 'none',
    'printer.softpath_html': 'none',
    'printer.path': 'none',
    'logging.level': 'INFO',
    'logging.output': 'stdout',
    'debug_mode_tooltips':False,
    'client.default_path': os.path.expanduser('~'),
    'support.recipient': 'support@openerp.com',
    'support.support_id' : '',
    'form.toolbar': True,
    'form.submenu': True,
    'client.form_tab': 'top',
    'client.form_tab_orientation': 0,
    'client.lang': False,
    'client.filetype': {},
    'client.timeout': 3600,
    'client.form_text_spellcheck': True,
}


addon_description = lambda addon_token: {
    'name': 'Dummy addon %s' % addon_token,
    'version': '0.0',
    'category': 'None',
    'description': 'Dummy addon.',
    'author': user,
    'website': 'http://www.openerp.com/',
    'depends': [],
    'init_xml': [],
    'update_xml': [],
    'demo_xml': [],
    'test': [],
    'installable': True,
    'active': False,
}

init_body = """
#
# Insert all includes necesary for this module
#
"""

default_files = {
    '%(root)s/etc/openerp-client.conf': """
[printer]
preview = True
softpath_html = None
softpath = None
path = None

[logging]
output = stdout
level = error

[tip]
position = 0
autostart = False

[path]
pixmaps = $(root)s/share/pixmaps/openerp-client
share = $(root)s/share/openerp-client

[client]
form_tab_orientation = 0
form_tab = top
lang = es_AR
theme = None
timeout = 3600
toolbar = both
filetype = {}
#default_path = /home/crocha/Proyectos/openerp/6.0/openerp-argentina/l10n_ar_invoice/i18n
form_text_spellcheck = True

[form]
autosave = False
toolbar = True
submenu = True

[login]
protocol = socket://
login = admin
port = 8070
server = localhost
db = polpor_erp_v6

[support]
support_id = 
recipient = support@openerp.com
""",
    '%(root)s/etc/openerp-server.conf': """
[options]
without_demo = True
; This is the password that allows database operations:
; admin_passwd = admin
upgrade = True
verbose = True
netrpc = True
; netrpc_interface = 
; netrpc_port = 
xmlrpc = True
; xmlrpc_interface = 
xmlrpc_port = 8069
db_host = False
db_port = False
; Please uncomment the following line *after* you have created the
; database. It activates the auto module check on startup.
db_name = ctmil
;db_user = openerp
;db_password = False
; Uncomment these for xml-rpc over SSL
; secure = True
; secure_cert_file = /etc/openerp/server.cert
; secure_pkey_file = /etc/openerp/server.key
root_path = None
soap = False
; translate_modules = ['all']
demo = {}
addons_path = None
reportgz = False

; Static http parameters
static_http_enable = False
static_http_document_root = /var/www/html
static_http_url_prefix = /
""",
    '%(root)s/etc/openerp-client.conf': """
[global]
addons-path=%(root)/addons
"""
}

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
